# Copyright (C) 2018 Pluralsight LLC

import unittest
from six import BytesIO

import spavro.io
from spavro.io import FastDatumWriter
# from spavro.io import SlowDatumWriter as FastDatumWriter
# from spavro.exceptions import AvroTypeException, SchemaParseException
from spavro.schema import SchemaParseException, AvroException, UnionSchema


from spavro import schema

invalid_schemas = (
    ("true_is_not_a_schema", '"True"'),
    ("unquoted_true_is_not_a_schema", 'True'),
    ("missing_type", '{"no_type": "test"}'),
    ("invalid_type", '{"type": "panther"}'),
    ("missing_required_size_in_fixed", '''{"type": "fixed",
         "name": "Missing size"}'''),
    ("missing_required_name_in_fixed", '''{"type": "fixed",
         "size": 314}'''),
    ("enum_symbols_not_an_array", '''{"type": "enum",
         "name": "Status",
         "symbols": "Normal Caution Critical"}'''),
    ("missing_required_name_in_enum", '''{"type": "enum",
         "symbols": ["Normal", "Caution", "Critical"]}''')
)

valid_schemas = (
    ("bytes_in_a_union",
        '["null", "string", "bytes"]',
        UnionSchema),
    ("bytes_and_fixed_in_a_union",
        '["bytes", {"name": "sixteen-bytes", "type": "fixed", "size": 16}]',
        UnionSchema),
    )


class TestSchemaParsing(unittest.TestCase):
    pass


def create_good_case(local_schema, expected_schema_type):
    def test_write_good_data(self):
        write_schema = spavro.schema.parse(local_schema)
        self.assertIs(type(write_schema), expected_schema_type)
    return test_write_good_data


def create_exception_case(local_schema):
    # print(schema)
    def test_parse_invalid_schema(self):
        with self.assertRaises((SchemaParseException, AvroException)) as context:
            spavro.schema.parse(local_schema)
    return test_parse_invalid_schema


def make_good_cases(cases):
    for name, local_schema, expected_schema_type in cases:
        test_method = create_good_case(local_schema, expected_schema_type)
        test_method.__name__ = 'test_good_schema_{}'.format(name)
        setattr(TestSchemaParsing, test_method.__name__, test_method)


def make_exception_cases(cases):
    for name, local_schema in cases:
        test_method = create_exception_case(local_schema)
        test_method.__name__ = 'test_invalid_schema_{}'.format(name)
        setattr(TestSchemaParsing, test_method.__name__, test_method)


make_good_cases(valid_schemas)
make_exception_cases(invalid_schemas)



# Copyright (C) 2017 Pluralsight LLC

import unittest
import six
from six import BytesIO as StringIO
# try:
#   from cStringIO import StringIO
# except ImportError:
#   from StringIO import StringIO
try:
    type(unicode)
except NameError:
    unicode = str

import json
import spavro.schema
import spavro.io

from spavro.io import FastDatumWriter


class TestMoreIo(unittest.TestCase):
    def setUp(self):
        self.fdw = FastDatumWriter()
        self.sdw = SlowDatumWriter()

# name, schema, data
cases = (
("null_and_list_in_union", [{"type": "array", "items": "string"}, "null"], None)
)


def create_case(schema, datum):
    def compare_old_and_new(self):
        buffer = StringIO()
        encoder = spavro.io.FastBinaryEncoder(fastbuff)
        write_schema = spavro.schema.parse(json.dumps(schema))
        for i in range(10):
            self.fdw.write_data(write_schema, datum, fastencoder)
            self.sdw.write_data(write_schema, datum, slowencoder)
        self.assertEqual(fastbuff.getvalue(), slowbuff.getvalue())
    return compare_old_and_new


def make_cases():
    for name, schema, datum in cases:
        test_method = create_case(schema, datum)
        test_method.__name__ = 'test_write_old_vs_new_{}'.format(name)
        setattr(TestOldVsNew, test_method.__name__, test_method)

make_cases()





PRIMITIVE = (
    u'null',
    u'boolean',
    u'string',
    u'bytes',
    u'int',
    u'long',
    u'float',
    u'double',
)


class Schema(object):
    def to_json(self):
        raise NotImplemented()

    def __str__(self):
        return str(self.type)

    def __repr__(self):
        return "<{} type='{}'>".format(self.__class__.__name__, self)


class PrimitiveSchema(Schema):
    def __init__(self, schema_name):
        self.type = schema_name


class RecordField(object):
    def __init__(self, fielddef):
        self.name = fielddef['name']
        self.type = parse_schema(fielddef['type'])

    def __str__(self):
        return str(self.type)

    def __repr__(self):
        return "<{} type='{}'>".format(self.__class__.__name__, self)


class RecordSchema(Schema):
    def __init__(self, schema):
        self.name = schema['name']
        self.type = schema['type']
        self.fields = [RecordField(field) for field in schema['fields']]


class UnionSchema(Schema):
    def __init__(self, schemas, names=None):
        self.type = 'union'
        self.schemas = [parse_schema(schema, names) for schema in schemas]


class EnumSchema(Schema):
    def __init__(self, schema):
        self.type = 'enum'
        self.symbols = schema['symbols']
        self.name = schema.get('name', None)


class ArraySchema(Schema):
    def __init__(self, schema):
        raise NotImplemented()


class MapSchema(Schema):
    def __init__(self, schema):
        raise NotImplemented()


class FixedSchema(Schema):
    def __init__(self, schema):
        raise NotImplemented()


# all complex types are represented by dictionaries
complex_types = {
    'record': RecordSchema,
    'enum': EnumSchema,
    'array': ArraySchema,
    'map': MapSchema,
    'fixed': FixedSchema
}


def parse_schema(schema, names=None):
    if type(schema) is list:
        return UnionSchema(schema)
    elif type(schema) is dict:
        if schema['type'] in complex_types:
            return complex_types[schema['type']](schema)
        elif schema['type'] in PRIMITIVE:
            # could add if 'logicalType' in schema as a double guard
            # this handles annotated schemas and logical types
            # ignores everything else in the dictionary
            return parse_schema(schema['type'])
    elif schema in PRIMITIVE:
        return PrimitiveSchema(schema)

    raise Exception("Invalid schema: {}".format(schema))
